@@ -801,8 +801,7 @@ auth_request_fix_username(struct auth_request *request, const char *username,
 		request->user = user;
 
 		dest = t_str_new(256);
-		table = auth_request_get_var_expand_table(request,
-						auth_request_str_escape);
+		table = auth_request_get_var_expand_table(request, NULL);
 		var_expand(dest, request->auth->username_format, table);
 		user = p_strdup(request->pool, str_c(dest));
 
@@ -1087,9 +1086,11 @@ void auth_request_set_field(struct auth_request *request,
 		return;
 	}
 
-	if (passdb_cache != NULL &&
-	    request->passdb->passdb->cache_key != NULL) {
-		/* we'll need to get this field stored into cache */
+	if ((passdb_cache != NULL &&
+	     request->passdb->passdb->cache_key != NULL) || worker) {
+		/* we'll need to get this field stored into cache,
+		   or we're a worker and we'll need to send this to the main
+		   auth process that can store it in the cache. */
 		if (request->extra_cache_fields == NULL) {
 			request->extra_cache_fields =
 				auth_stream_reply_init(request->pool);
