@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2008 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2005-2009 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -100,7 +100,8 @@ static void auth_parse_input(struct auth_connection *conn, const char *args)
 	bool debug = getenv("DEBUG") != NULL;
 	unsigned int len;
 
-	for (tmp = t_strsplit(args, "\t"); *tmp != NULL; tmp++) {
+	tmp = t_strsplit(args, "\t");
+	for (tmp++; *tmp != NULL; tmp++) {
 		if (debug)
 			i_info("auth input: %s", *tmp);
 
