@@ -682,7 +682,7 @@ static void open_logfile(const char *username)
 	const char *prefix, *log_path, *stamp;
 
 	prefix = t_strdup_printf("deliver(%s)", username);
-	log_path = getenv("LOG_PATH");
+	log_path = home_expand(getenv("LOG_PATH"));
 	if (log_path == NULL || *log_path == '\0') {
 		const char *env = getenv("SYSLOG_FACILITY");
 		int facility;
@@ -695,7 +695,7 @@ static void open_logfile(const char *username)
 		i_set_failure_file(log_path, t_strconcat(prefix, ": ", NULL));
 	}
 
-	log_path = getenv("INFO_LOG_PATH");
+	log_path = home_expand(getenv("INFO_LOG_PATH"));
 	if (log_path != NULL && *log_path != '\0')
 		i_set_info_file(log_path);
 
@@ -1018,6 +1018,9 @@ int main(int argc, char *argv[])
 	deliver_set->sendmail_path = getenv("SENDMAIL_PATH");
 	if (deliver_set->sendmail_path == NULL)
 		deliver_set->sendmail_path = DEFAULT_SENDMAIL_PATH;
+	deliver_set->rejection_subject = getenv("REJECTION_SUBJECT");
+	if (deliver_set->rejection_subject == NULL)
+		deliver_set->rejection_subject = DEFAULT_MAIL_REJECTION_SUBJECT;
 	deliver_set->rejection_reason = getenv("REJECTION_REASON");
 	if (deliver_set->rejection_reason == NULL) {
 		deliver_set->rejection_reason =
