@@ -46,6 +46,7 @@ struct pgsql_result {
 	struct sql_result api;
 	PGresult *pgres;
 
+	char *query;
 	unsigned int rownum, rows;
 	unsigned int fields_count;
 	const char **fields;
@@ -55,6 +56,8 @@ struct pgsql_result {
 
 	sql_query_callback_t *callback;
 	void *context;
+
+	unsigned int retry_query:1;
 };
 
 struct pgsql_queue {
@@ -80,18 +83,26 @@ struct pgsql_transaction_context {
 extern struct sql_db driver_pgsql_db;
 extern struct sql_result driver_pgsql_result;
 
+static void
+driver_pgsql_query_full(struct sql_db *db, const char *query,
+			sql_query_callback_t *callback, void *context,
+			bool retry_query);
 static void queue_send_next(struct pgsql_db *db);
 static void result_finish(struct pgsql_result *result);
 
 static void driver_pgsql_close(struct pgsql_db *db)
 {
-	if (db->io != NULL)
-		io_remove(&db->io);
 	db->io_dir = 0;
 
 	PQfinish(db->pg);
 	db->pg = NULL;
 
+	if (db->io != NULL) {
+		/* The fd may be closed before call to PQfinish() already,
+		   so use io_remove_closed(). */
+		io_remove_closed(&db->io);
+	}
+
 	db->connecting = FALSE;
 	db->connected = FALSE;
         db->querying = FALSE;
@@ -276,6 +287,7 @@ static void driver_pgsql_result_free(struct sql_result *_result)
 
 	i_free(result->fields);
 	i_free(result->values);
+	i_free(result->query);
 	i_free(result);
 
 	if (db->queue != NULL && !db->querying && db->connected)
@@ -285,9 +297,15 @@ static void driver_pgsql_result_free(struct sql_result *_result)
 static void result_finish(struct pgsql_result *result)
 {
 	struct pgsql_db *db = (struct pgsql_db *)result->api.db;
-	bool free_result = TRUE;
+	bool free_result = TRUE, retry = FALSE;
+	bool disconnected;
 
-	if (result->callback != NULL) {
+	disconnected = PQstatus(db->pg) == CONNECTION_BAD;
+	if (disconnected && result->pgres == NULL && result->retry_query) {
+		/* retry the query */
+		i_error("pgsql: Query failed, retrying: %s", last_error(db));
+		retry = TRUE;
+	} else if (result->callback != NULL) {
 		result->api.callback = TRUE;
 		T_BEGIN {
 			result->callback(&result->api, result->context);
@@ -295,13 +313,20 @@ static void result_finish(struct pgsql_result *result)
 		result->api.callback = FALSE;
 		free_result = db->sync_result != &result->api;
 	}
-	if (free_result)
-		driver_pgsql_result_free(&result->api);
 
-	if (PQstatus(db->pg) == CONNECTION_BAD) {
+	if (disconnected) {
 		/* disconnected */
 		driver_pgsql_close(db);
+
+		if (retry) {
+			/* retry the query */
+			driver_pgsql_query_full(&db->api, result->query,
+						result->callback,
+						result->context, FALSE);
+		}
 	}
+	if (free_result)
+		driver_pgsql_result_free(&result->api);
 }
 
 static void get_result(struct pgsql_result *result)
@@ -491,10 +516,10 @@ static void do_query(struct pgsql_result *result, const char *query)
 	}
 }
 
-static void exec_callback(struct sql_result *result,
+static void exec_callback(struct sql_result *_result,
 			  void *context ATTR_UNUSED)
 {
-        struct pgsql_db *db = (struct pgsql_db *)result->db;
+	struct pgsql_db *db = (struct pgsql_db *)_result->db;
 
 	i_error("pgsql: sql_exec() failed: %s", last_error(db));
 }
@@ -521,7 +546,8 @@ driver_pgsql_escape_string(struct sql_db *_db, const char *string)
 	return to;
 }
 
-static void driver_pgsql_exec(struct sql_db *db, const char *query)
+static void driver_pgsql_exec_full(struct sql_db *db, const char *query,
+				   bool retry_query)
 {
 	struct pgsql_result *result;
 
@@ -529,12 +555,22 @@ static void driver_pgsql_exec(struct sql_db *db, const char *query)
 	result->api = driver_pgsql_result;
 	result->api.db = db;
 	result->callback = exec_callback;
-
+	if (retry_query) {
+		result->query = i_strdup(query);
+		result->retry_query = TRUE;
+	}
 	do_query(result, query);
 }
 
-static void driver_pgsql_query(struct sql_db *db, const char *query,
-			       sql_query_callback_t *callback, void *context)
+static void driver_pgsql_exec(struct sql_db *db, const char *query)
+{
+	driver_pgsql_exec_full(db, query, TRUE);
+}
+
+static void
+driver_pgsql_query_full(struct sql_db *db, const char *query,
+			sql_query_callback_t *callback, void *context,
+			bool retry_query)
 {
 	struct pgsql_result *result;
 
@@ -543,10 +579,19 @@ static void driver_pgsql_query(struct sql_db *db, const char *query,
 	result->api.db = db;
 	result->callback = callback;
 	result->context = context;
-
+	if (retry_query) {
+		result->query = i_strdup(query);
+		result->retry_query = TRUE;
+	}
 	do_query(result, query);
 }
 
+static void driver_pgsql_query(struct sql_db *db, const char *query,
+			       sql_query_callback_t *callback, void *context)
+{
+	driver_pgsql_query_full(db, query, callback, context, TRUE);
+}
+
 static void pgsql_query_s_callback(struct sql_result *result, void *context)
 {
         struct pgsql_db *db = context;
@@ -772,16 +817,21 @@ static const char *driver_pgsql_result_get_error(struct sql_result *_result)
 	const char *msg;
 	size_t len;
 
-	msg = PQresultErrorMessage(result->pgres);
-	if (msg == NULL)
-		return "(no error set)";
+	i_free_and_null(db->error);
 
-	/* Error message should contain trailing \n, we don't want it */
-	len = strlen(msg);
-	i_free(db->error);
-	db->error = len == 0 || msg[len-1] != '\n' ?
-		i_strdup(msg) : i_strndup(msg, len-1);
+	if (result->pgres == NULL) {
+		/* connection error */
+		db->error = i_strdup(last_error(db));
+	} else {
+		msg = PQresultErrorMessage(result->pgres);
+		if (msg == NULL)
+			return "(no error set)";
 
+		/* Error message should contain trailing \n, we don't want it */
+		len = strlen(msg);
+		db->error = len == 0 || msg[len-1] != '\n' ?
+			i_strdup(msg) : i_strndup(msg, len-1);
+	}
 	return db->error;
 }
 
@@ -831,7 +881,8 @@ driver_pgsql_transaction_commit(struct sql_transaction_context *_ctx,
 	ctx->callback = callback;
 	ctx->context = context;
 
-	sql_query(_ctx->db, "COMMIT", transaction_commit_callback, ctx);
+	driver_pgsql_query_full(_ctx->db, "COMMIT",
+				transaction_commit_callback, ctx, FALSE);
 }
 
 static int
@@ -844,7 +895,7 @@ driver_pgsql_transaction_commit_s(struct sql_transaction_context *_ctx,
 
 	if (ctx->failed) {
 		*error_r = ctx->error;
-		if (!ctx->opened)
+		if (ctx->opened)
 			sql_exec(_ctx->db, "ROLLBACK");
 	} else if (!ctx->opened)
 		*error_r = NULL;
@@ -875,9 +926,10 @@ driver_pgsql_transaction_rollback(struct sql_transaction_context *_ctx)
 }
 
 static void
-transaction_update_callback(struct sql_result *result,
-			    struct pgsql_transaction_context *ctx)
+transaction_update_callback(struct sql_result *result, void *context)
 {
+	struct pgsql_transaction_context *ctx = context;
+
 	if (sql_result_next_row(result) < 0) {
 		ctx->failed = TRUE;
 		ctx->error = sql_result_get_error(result);
@@ -898,7 +950,8 @@ driver_pgsql_update(struct sql_transaction_context *_ctx, const char *query)
 		sql_query(_ctx->db, "BEGIN", transaction_update_callback, ctx);
 	}
 
-	sql_query(_ctx->db, query, transaction_update_callback, ctx);
+	driver_pgsql_query_full(_ctx->db, query,
+				transaction_update_callback, ctx, FALSE);
 }
 
 struct sql_db driver_pgsql_db = {
