@@ -317,6 +317,9 @@ static int mkdir_verify(struct mail_storage *storage,
 	} else if (errno == ENOENT) {
 		mail_storage_set_error(storage, MAIL_ERROR_NOTFOUND,
 			"Mailbox was deleted while it was being created");
+	} else if (errno == EACCES) {
+		mail_storage_set_critical(storage, "%s",
+			mail_storage_create_eacces_msg("mkdir", dir));
 	} else {
 		mail_storage_set_critical(storage,
 					  "mkdir(%s) failed: %m", dir);
@@ -462,7 +465,7 @@ maildir_open(struct maildir_storage *storage, const char *name,
 
 	if (access(t_strconcat(path, "/cur", NULL), W_OK) < 0 &&
 	    errno == EACCES)
-		mbox->ibox.readonly = TRUE;
+		mbox->ibox.backend_readonly = TRUE;
 
 	mbox->keywords = maildir_keywords_init(mbox);
 	return &mbox->ibox.box;
