@@ -1,4 +1,4 @@
-/* Copyright (c) 2003-2008 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2003-2009 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "ioloop.h"
@@ -25,7 +25,6 @@ static void mbox_prepare_resync(struct index_mail *mail)
 		if (mbox->mbox_lock_id == t->mbox_lock_id)
 			t->mbox_lock_id = 0;
 		(void)mbox_unlock(mbox, mbox->mbox_lock_id);
-		mbox->mbox_lock_id = 0;
 		i_assert(mbox->mbox_lock_type == F_UNLCK);
 	}
 }
@@ -49,10 +48,16 @@ static int mbox_mail_seek(struct index_mail *mail)
 	}
 
 	for (try = 0; try < 2; try++) {
+		if ((sync_flags & MBOX_SYNC_FORCE_SYNC) != 0) {
+			/* dirty offsets are broken. make sure we can sync. */
+			mbox_prepare_resync(mail);
+		}
 		if (mbox->mbox_lock_type == F_UNLCK) {
 			sync_flags |= MBOX_SYNC_LOCK_READING;
 			if (mbox_sync(mbox, sync_flags) < 0)
 				return -1;
+			t->mbox_lock_id = mbox->mbox_lock_id;
+			i_assert(t->mbox_lock_id != 0);
 
 			/* refresh index file after mbox has been locked to
 			   make sure we get only up-to-date mbox offsets. */
@@ -62,13 +67,13 @@ static int mbox_mail_seek(struct index_mail *mail)
 			}
 
 			i_assert(mbox->mbox_lock_type != F_UNLCK);
-			t->mbox_lock_id = mbox->mbox_lock_id;
-		} else if ((sync_flags & MBOX_SYNC_FORCE_SYNC) != 0) {
-			/* dirty offsets are broken and mbox is write-locked.
-			   sync it to update offsets. */
-			mbox_prepare_resync(mail);
-			if (mbox_sync(mbox, sync_flags) < 0)
-				return -1;
+		} else if (t->mbox_lock_id == 0) {
+			/* file is already locked by another transaction, but
+			   we must keep it locked for the entire transaction,
+			   so increase the lock counter. */
+			if (mbox_lock(mbox, mbox->mbox_lock_type,
+				      &t->mbox_lock_id) < 0)
+				i_unreached();
 		}
 
 		if (mbox_file_open_stream(mbox) < 0)
@@ -189,10 +194,12 @@ mbox_mail_get_next_offset(struct index_mail *mail, uoff_t *next_offset_r)
 	}
 
 	/* We can't really trust trans_view. The next message may already be
-	   expugned from it. hdr.sync_size may also be updated, but
-	   hdr.messages_count not. So refresh the index to get the latest
-	   changes and get the next message's offset using a new view. */
-	(void)mail_index_refresh(mail->ibox->index);
+	   expunged from it. Also hdr.messages_count may be incorrect there.
+	   So refresh the index to get the latest changes and get the next
+	   message's offset using a new view. */
+	i_assert(mbox->mbox_lock_type != F_UNLCK);
+	if (mbox_sync_header_refresh(mbox) < 0)
+		return -1;
 
 	view = mail_index_view_open(mail->ibox->index);
 	hdr = mail_index_get_header(view);
@@ -203,7 +210,7 @@ mbox_mail_get_next_offset(struct index_mail *mail, uoff_t *next_offset_r)
 		/* last message, use the synced mbox size */
 		trailer_size = (mbox->storage->storage.flags &
 				MAIL_STORAGE_FLAG_SAVE_CRLF) != 0 ? 2 : 1;
-		*next_offset_r = hdr->sync_size - trailer_size;
+		*next_offset_r = mbox->mbox_hdr.sync_size - trailer_size;
 	} else {
 		if (mbox_file_lookup_offset(mbox, view, seq + 1,
 					    next_offset_r) <= 0)
