@@ -98,7 +98,7 @@ static int cmd_dele(struct client *client, const char *args)
 	unsigned int msgnum;
 
 	if (get_msgnum(client, args, &msgnum) == NULL)
-		return 0;
+		return -1;
 
 	if (!client->deleted) {
 		client->deleted_bitmask = i_malloc(MSGS_BITMASK_SIZE(client));
@@ -161,7 +161,7 @@ static int cmd_list(struct client *client, const char *args)
 		unsigned int msgnum;
 
 		if (get_msgnum(client, args, &msgnum) == NULL)
-			return 0;
+			return -1;
 
 		client_send_line(client, "+OK %u %"PRIuUOFF_T, msgnum+1,
 				 client->message_sizes[msgnum]);
@@ -365,9 +365,23 @@ static void fetch_callback(struct client *client)
 	client->cmd = NULL;
 }
 
-static void fetch(struct client *client, unsigned int msgnum, uoff_t body_lines)
+static int client_reply_msg_expunged(struct client *client, unsigned int msgnum)
+{
+	client_send_line(client, "-ERR Message %u expunged.", msgnum + 1);
+	if (msgnum <= client->highest_expunged_fetch_msgnum) {
+		/* client tried to fetch an expunged message again.
+		   treat this as error so we'll eventually disconnect the
+		   client instead of letting it loop forever. */
+		return -1;
+	}
+	client->highest_expunged_fetch_msgnum = msgnum;
+	return 1;
+}
+
+static int fetch(struct client *client, unsigned int msgnum, uoff_t body_lines)
 {
         struct fetch_context *ctx;
+	int ret;
 
 	ctx = i_new(struct fetch_context, 1);
 
@@ -382,9 +396,9 @@ static void fetch(struct client *client, unsigned int msgnum, uoff_t body_lines)
 
 	if (mailbox_search_next(ctx->search_ctx, ctx->mail) <= 0 ||
 	    mail_get_stream(ctx->mail, NULL, NULL, &ctx->stream) < 0) {
-		client_send_line(client, "-ERR Message not found.");
+		ret = client_reply_msg_expunged(client, msgnum);
 		fetch_deinit(ctx);
-		return;
+		return ret;
 	}
 
 	if (body_lines == (uoff_t)-1 && !no_flag_updates) {
@@ -407,6 +421,7 @@ static void fetch(struct client *client, unsigned int msgnum, uoff_t body_lines)
 	client->cmd = fetch_callback;
 	client->cmd_context = ctx;
 	fetch_callback(client);
+	return 1;
 }
 
 static int cmd_retr(struct client *client, const char *args)
@@ -414,7 +429,7 @@ static int cmd_retr(struct client *client, const char *args)
 	unsigned int msgnum;
 
 	if (get_msgnum(client, args, &msgnum) == NULL)
-		return 0;
+		return -1;
 
 	if (client->last_seen <= msgnum)
 		client->last_seen = msgnum+1;
@@ -423,8 +438,7 @@ static int cmd_retr(struct client *client, const char *args)
 	client->byte_counter = &client->retr_bytes;
 	client->byte_counter_offset = client->output->offset;
 
-	fetch(client, msgnum, (uoff_t)-1);
-	return 1;
+	return fetch(client, msgnum, (uoff_t)-1);
 }
 
 static int cmd_rset(struct client *client, const char *args ATTR_UNUSED)
@@ -485,16 +499,15 @@ static int cmd_top(struct client *client, const char *args)
 
 	args = get_msgnum(client, args, &msgnum);
 	if (args == NULL)
-		return 0;
+		return -1;
 	if (get_size(client, args, &max_lines) == NULL)
-		return 0;
+		return -1;
 
 	client->top_count++;
 	client->byte_counter = &client->top_bytes;
 	client->byte_counter_offset = client->output->offset;
 
-	fetch(client, msgnum, max_lines);
-	return 1;
+	return fetch(client, msgnum, max_lines);
 }
 
 struct cmd_uidl_context {
@@ -588,7 +601,7 @@ static bool list_uids_iter(struct client *client, struct cmd_uidl_context *ctx)
 		if (ret == 0 && ctx->message == 0) {
 			/* output is being buffered, continue when there's
 			   more space */
-			return 0;
+			return FALSE;
 		}
 	}
 
@@ -650,16 +663,16 @@ static int cmd_uidl(struct client *client, const char *args)
 	if (*args == '\0') {
 		client_send_line(client, "+OK");
 		ctx = cmd_uidl_init(client, 0);
-		list_uids_iter(client, ctx);
+		(void)list_uids_iter(client, ctx);
 	} else {
 		unsigned int msgnum;
 
 		if (get_msgnum(client, args, &msgnum) == NULL)
-			return 0;
+			return -1;
 
 		ctx = cmd_uidl_init(client, msgnum+1);
 		if (!list_uids_iter(client, ctx))
-			client_send_line(client, "-ERR Message not found.");
+			return client_reply_msg_expunged(client, msgnum);
 	}
 
 	return 1;
